package cron

import (
	"context"
	"time"

	"github.com/sirupsen/logrus"
)

type Cron struct {
	time   time.Time
	ctx    context.Context
	cancel context.CancelFunc
	f      func()
	log    *logrus.Entry
}

func NewCron() *Cron {
	now := time.Now()
	c := &Cron{
		time: time.Date(now.Year(), now.Month(), now.Day(), 01, 0, 0, 0, now.Location()),
		log:  logrus.New().WithField("part", "crontab"),
	}
	return c
}

func (c *Cron) SetFunc(f func()) *Cron {
	c.f = f
	return c
}

func (c *Cron) SetContext(ctx context.Context) *Cron {
	c.ctx = ctx
	return c
}

func (c *Cron) SetTimeHHmm(HHmm string) *Cron {
	t, _ := time.Parse("15:04", HHmm)
	now := time.Now()
	c.time = time.Date(now.Year(), now.Month(), now.Day(), t.Hour(), t.Minute(), 0, 0, now.Location())
	return c
}

func (c *Cron) Run(ctx context.Context) {
	c.ctx, c.cancel = context.WithCancel(ctx)
	for {
		if time.Now().After(c.time) {
			c.time = c.time.Add(time.Hour * 24)
		}
		c.log.Infof("cron 下次运行时间: %s", c.time)
		select {
		case <-time.After(time.Until(c.time)):
			if c.f != nil {
				c.f()
			}
		case <-c.ctx.Done():
			return
		}
	}
}

func (c *Cron) Stop() {
	c.cancel()
}
